package envoy.client.event;

import java.io.IOException;
import java.util.logging.ConsoleHandler;
import java.util.logging.FileHandler;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.LogRecord;
import java.util.logging.Logger;
import java.util.logging.SimpleFormatter;

/**
 * Project: <strong>envoy-client</strong><br>
 * File: <strong>EnvoyLogger.java</strong><br>
 * Created: <strong>14 Dec 2019</strong><br>
 * 
 * @author Leon Hofmeister
 * @since Envoy v0.2-alpha
 */
public class EnvoyLogger extends Logger {

	private Logger	logger;
	private int		fileLevel	= 800;

	private Handler handler = new Handler() {

		@Override
		public void publish(LogRecord arg0) {
			ConsoleHandler	ch;
			FileHandler		fh;
			SimpleFormatter	formatter	= new SimpleFormatter();
			if (arg0.getLevel().intValue() >= fileLevel) {// Case if level >= info
				try {
					fh = new FileHandler("Envoy_user.log");
					logger.addHandler(fh);
					formatter.formatMessage(arg0);
					fh.setFormatter(formatter);
				} catch (SecurityException | IOException e) {
					e.printStackTrace();
				}
			}
			ch = new ConsoleHandler();
			logger.addHandler(ch);
			formatter.formatMessage(arg0);
		}

		@Override
		public void flush() {}

		@Override
		public void close() throws SecurityException {}
	};

	public EnvoyLogger(String name) {
		super(name, null);
		logger.addHandler(handler);
	}

	/**
	 * Logs a message. If the problem severity is above the FileLevel-barrier, then
	 * the log entry will be written to a file. Regardless of problem severity,
	 * everything will be printed to the console.
	 * 
	 * @param level the problem severity
	 * @param msg   the message to be written in the log
	 * @since Envoy v0.2-alpha
	 */
	@Override
	public void log(Level level, String msg) {
		LogRecord lr = new LogRecord(level, msg);
		logger.log(lr);
	}

	/**
	 * Logs a message. If the problem severity is above the {@code FileLevel}
	 * barrier, then the log entry will be written to a file.
	 * Regardless of problem severity, everything will be printed to the console.
	 * 
	 * @param logRecord the LogRecord (Level and String) to be treated by the
	 *                  Logger.
	 * @since Envoy v0.2-alpha
	 */
	@Override
	public void log(LogRecord logRecord) { logger.log(logRecord); }

	/**
	 * @return the fileLevel: The current barrier for writing logs to a file. It can
	 *         range from 100-1000 in steps of one hundred with 1000 being
	 *         Level.SEVERE
	 * @since Envoy v0.2-alpha
	 */
	public int getFileLevel() { return fileLevel; }

	/**
	 * @param fileLevel the severity above which on logRecords will be written in a
	 *                  file instead of the console
	 * @since Envoy v0.2-alpha
	 */
	public void setFileLevel(int fileLevel) {
		if (fileLevel <= 10) fileLevel *= 100;
		this.fileLevel = fileLevel;
	}
}
